// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KCALUTILS_VERSION_H
#define KCALUTILS_VERSION_H

#define KCALUTILS_VERSION_STRING "5.6.3"
#define KCALUTILS_VERSION_MAJOR 5
#define KCALUTILS_VERSION_MINOR 6
#define KCALUTILS_VERSION_PATCH 3
#define KCALUTILS_VERSION ((5<<16)|(6<<8)|(3))

#endif
